﻿using System;
using System.Globalization;
using System.Windows.Forms;
using TaiTanXing.Ctp.CtpNet;

namespace CtpNetCsDemo
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// CtpNet
        /// </summary>
        private CtpNetClass ctpNet;

        #region 辅助函数

        /// <summary>
        /// 清空函数日志
        /// </summary>
        private void Button_ClearFunctionLog_Click(object sender, EventArgs e)
        {
            TextBox_Log.Clear();
        }

        /// <summary>
        /// 清空事件日志
        /// </summary>
        private void Button_ClearEventLog_Click(object sender, EventArgs e)
        {
            TextBox_EventLog.Clear();
        }

        /// <summary>
        /// 窗体加载
        /// </summary>
        private void MainForm_Load(object sender, EventArgs e)
        {
        }

        /// <summary>
        /// 窗体已关闭
        /// </summary>
        private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            //清理组件
            if (ctpNet != null)
            {
                try
                {
                    ctpNet.Release();
                }
                catch
                {
                    // ignored
                }
            }

            Environment.Exit(0);
        }

        /// <summary>
        /// 检查是否已登录
        /// </summary>
        private bool CheckConnection()
        {
            if (ctpNet == null || ctpNet.IsTradeServerLogined == false)
            {
                Log("请先登录");
                return false;
            }

            return true;
        }

        /// <summary>
        /// 获取窗体界面上输入的合约代码
        /// </summary>
        private string GetInstrumentID()
        {
            return TextBox_InstrumentID.Text.Trim();
        }

        /// <summary>
        /// 获取窗体界面上输入的报单数量,失败返回 -1
        /// </summary>
        /// <returns></returns>
        private int GetVolume()
        {
            var str = TextBox_Volume.Text.Trim();
            if (string.IsNullOrWhiteSpace(str))
            {
                Log("请输入报单数量");
                TextBox_Volume.Clear();
                TextBox_Volume.Focus();
                return -1;
            }

            if (int.TryParse(str, out var intVolume) == false || intVolume < 1)
            {
                Log("输入的报单数量无效:" + str);
                TextBox_Volume.Clear();
                TextBox_Volume.Focus();
                return -1;
            }

            return intVolume;
        }

        /// <summary>
        /// 获取窗体界面上输入的报单价格,失败返回 Double.NaN
        /// </summary>
        /// <returns></returns>
        private double GetPrice()
        {
            var str = TextBox_Price.Text.Trim();
            if (string.IsNullOrWhiteSpace(str))
            {
                Log("请输入报单价格");
                TextBox_Price.Clear();
                TextBox_Price.Focus();
                return double.NaN;
            }

            if (double.TryParse(str, out var dblPrice) == false)
            {
                Log("输入的报单价格无效:" + str);
                TextBox_Price.Clear();
                TextBox_Price.Focus();
                return double.NaN;
            }

            return dblPrice;
        }

        /// <summary>
        /// 获取窗体界面上输入的持仓方向
        /// </summary>
        /// <returns></returns>
        private CtpNetDirectionType GetDirection()
        {
            if (radioButton_Buy.Checked) return CtpNetDirectionType.Buy;
            if (radioButton_Sell.Checked) return CtpNetDirectionType.Sell;
            return 0; //不指定
        }

        /// <summary>
        /// 获取窗体界面上输入的开平标志
        /// </summary>
        /// <returns></returns>
        private CtpNetOffsetFlagType GetOffsetFlag()
        {
            if (radioButton_Open.Checked) return CtpNetOffsetFlagType.Open;
            if (radioButton_Close.Checked) return CtpNetOffsetFlagType.Close;
            if (radioButton_CloseYesterday.Checked) return CtpNetOffsetFlagType.CloseYesterday;
            if (radioButton_CloseToday.Checked) return CtpNetOffsetFlagType.CloseToday;
            return 0; //不指定
        }

        /// <summary>
        /// 获取窗体界面上输入的报单序号,失败返回 -1
        /// </summary>
        /// <returns></returns>
        private int GetOrderSn()
        {
            var str = TextBox_OrderSn.Text.Trim();
            if (string.IsNullOrWhiteSpace(str))
            {
                Log("请输入报单序号");
                TextBox_OrderSn.Clear();
                TextBox_OrderSn.Focus();
                return -1;
            }

            if (int.TryParse(str, out var intOrderSn) == false || intOrderSn < 1)
            {
                Log("输入的报单序号无效:" + str);
                TextBox_OrderSn.Clear();
                TextBox_OrderSn.Focus();
                return -1;
            }

            return intOrderSn;
        }
        
        /// <summary>
        /// 添加日志
        /// </summary>
        /// <param name="txt">日志内容</param>
        private void Log(string txt)
        {
            if (string.IsNullOrWhiteSpace(txt)) return;
            if (IsDisposed || Disposing) return;

            try
            {
                if (InvokeRequired)
                {
                    var a = new Action<string>(Log);
                    Invoke(a, txt);
                    return;
                }

                txt = txt.Trim();
                TextBox_Log.AppendText($"{DateTime.Now:yyyy-MM-dd HH:mm:ss.fff} {txt}{Environment.NewLine}");
            }
            catch
            {
                // ignored
            }
        }

        /// <summary>
        /// 添加事件日志
        /// </summary>
        /// <param name="txt">日志内容</param>
        private void LogEvent(string txt)
        {
            if (string.IsNullOrWhiteSpace(txt)) return;
            if (IsDisposed || Disposing) return;

            try
            {
                if (InvokeRequired)
                {
                    var a = new Action<string>(LogEvent);
                    Invoke(a, txt);
                    return;
                }

                txt = txt.Trim();
                TextBox_EventLog.AppendText($"{DateTime.Now:yyyy-MM-dd HH:mm:ss.fff} {txt}{Environment.NewLine}");
            }
            catch
            {
                // ignored
            }
        }

        #endregion 辅助函数

        #region 调用接口功能

        /// <summary>
        /// 注册CtpNet事件
        /// </summary>
        private void RegCtpNetEvent()
        {
            if (ctpNet == null) return;

            //程序日志通知
            ctpNet.OnLog += CtpNet_OnLog;
            //交易通知
            ctpNet.OnTradingNotice += CtpNet_OnTradingNotice;

            //行情服务器连接响应
            ctpNet.OnMarketDataServerConnected += CtpNet_OnMarketDataServerConnected;
            //行情服务器断开响应
            ctpNet.OnMarketDataServerDisconnected += CtpNet_OnMarketDataServerDisconnected;
            //交易服务器连接响应
            ctpNet.OnTradeServerConnected += CtpNet_OnTradeServerConnected;
            //交易服务器断开响应
            ctpNet.OnTradeServerDisconnected += CtpNet_OnTradeServerDisconnected;

            //初始化响应
            ctpNet.OnInitialized += CtpNetOnInitialized;
            //清理响应
            ctpNet.OnReleased += OnReleased;

            //行情通知
            ctpNet.OnMarketData += CtpNet_OnMarketData;
            //合约状态通知
            ctpNet.OnInstrumentStatus += CtpNet_OnInstrumentStatus;
            //资金变化通知
            ctpNet.OnTradingAccount += CtpNet_OnTradingAccount;

            //报单通知
            ctpNet.OnOrder += CtpNet_OnOrder;
            //成效通知
            ctpNet.OnTrade += CtpNet_OnTrade;
            //持仓变化通知
            ctpNet.OnPosition += CtpNet_OnPosition;
            
            //查询手续费率响应
            ctpNet.OnQueryCommissionRate += CtpNet_OnQueryCommissionRate;
            //查询保证金率响应
            ctpNet.OnQueryMarginRate += CtpNet_OnQueryMarginRate;
            //查询报单手续费率响应
            ctpNet.OnQueryOrderCommissionRate += CtpNet_OnQueryOrderCommissionRate;

            //更新交易密码响应
            ctpNet.OnUpdateUserPassword += CtpNet_OnUpdateUserPassword;
            //更新资金密码响应
            ctpNet.OnUpdateTradingAccountPassword += CtpNet_OnUpdateTradingAccountPassword;

            //查询银行余额响应
            ctpNet.OnQueryBankAccountMoney += CtpNet_OnQueryBankAccountMoney;
            //入金,银行资金转期货响应
            ctpNet.OnFromBankToFuture += CtpNet_OnFromBankToFuture;
            //出金,期货资金转银行响应
            ctpNet.OnFromFutureToBank += CtpNet_OnFromFutureToBank;
        }

        /// <summary>
        /// 登录
        /// </summary>
        private void Button_Login_Click(object sender, EventArgs e)
        {
            Log("发送登录请求");

            //清理旧组件
            if (ctpNet != null)
            {
                ctpNet.Release();
                ctpNet = null;
            }

            //创建组件实例
            ctpNet = new CtpNetClass();

            //注册CtpNet事件
            RegCtpNetEvent();

            var loginInfo = new CtpNetReqUserLoginFieldEx();

            //从快期的配置文件中读取
            //var responseCode2 = loginInfo.ReadConfigFile(@"快期的配置文件路径", "服务器名称");
            //if (responseCode2 != CtpNetResponseCode.NONE)
            //{
            //    Log("读取快期的配置文件失败:" + responseCode2.ToNameString());
            //    return;
            //}

            loginInfo.UserID = "034180";//TODO:
            loginInfo.Password = "123";//TODO:
            loginInfo.BrokerID = "9999";//TODO:

            loginInfo.MarketDataServerAddress.Add("180.168.146.187:10010");//可以添加更多的服务器地址
            loginInfo.TradeServerAddress.Add("180.168.146.187:10000");//可以添加更多的服务器地址
            //
            //loginInfo.MarketDataServerAddress.Add("180.168.146.187:10031");//TODO:
            //loginInfo.TradeServerAddress.Add("180.168.146.187:10030");//TODO:
            
            loginInfo.AuthCode = "";
            loginInfo.UserProductInfo = "";
            //等待初始化完成的时间(秒), 小于1秒,表示以异步方式登录,函数立即返回,可以在初始化完成事件中得到结果; 
            //大于等于1秒,表示以同步方式登录,函数直到初始化完成或者超时才返回
            //loginInfo.WaitTime = 30;

            var responseCode = ctpNet.Login(loginInfo);
            if (responseCode != CtpNetResponseCode.NONE)
            {
                Log("发送登录请求失败:" + responseCode.ToNameString());
                return;
            }

            Log("发送登录请求成功");
        }
        
        /// <summary>
        /// 获取交易日
        /// </summary>
        private void Button_GetTradingDay_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            Log("交易日:" + ctpNet.TradingDay);
        }

        /// <summary>
        /// 获取合约信息
        /// </summary>
        private void Button_GetInstrumentInfo_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var dic = ctpNet.GetInstruments();
            Log("合约数量:" + dic.Count);
            foreach (var instrument in dic.Values)
            {
                Log(instrument.InstrumentID + " " + instrument.InstrumentName);
            }
        }

        /// <summary>
        /// 获取合约状态
        /// </summary>
        private void Button_GetInstrumentStatus_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID)) return;

            var status = ctpNet.GetInstrumentStatus(instrumentID);
            if (status == null)
            {
                Log("获取 " + instrumentID + " 的状态失败");
                return;
            }

            Log("合约状态 " + instrumentID + " " + status.InstrumentStatus.ToNameString());
        }

        /// <summary>
        /// 订阅行情
        /// </summary>
        private void Button_SubscribeMarketData_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID)) return;

            var responseCode = ctpNet.SubscribeMarketData(instrumentID);
            if (responseCode != CtpNetResponseCode.NONE)
            {
                Log("订阅 " + instrumentID + " 的行情发送失败:" + responseCode.ToNameString());
                return;
            }

            Log("订阅 " + instrumentID + " 的行情发送成功");
        }

        /// <summary>
        /// 获取行情
        /// </summary>
        private void Button_GetMarketData_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID)) return;

            var tick = ctpNet.GetMarketData(instrumentID);
            if (tick == null)
            {
                Log("获取 " + instrumentID + " 的行情失败");
                return;
            }

            Log(instrumentID + " 的行情:" +
                "最新价=" + tick.LastPrice.ToString(CultureInfo.InvariantCulture) +
                ",卖一价=" + tick.AskPrice1.ToString(CultureInfo.InvariantCulture) +
                ",买一价=" + tick.BidPrice1.ToString(CultureInfo.InvariantCulture) +
                ",行情时间=" + tick.UpdateTime);
        }

        /// <summary>
        /// 获取帐号资金
        /// </summary>
        private void Button_GetAccountMoney_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            Log("动态权益=" + ctpNet.TradingAccount.Balance.ToString(CultureInfo.InvariantCulture) +
                ",可用资金=" + ctpNet.TradingAccount.Available.ToString(CultureInfo.InvariantCulture));
        }

        /// <summary>
        /// 多头开仓
        /// </summary>
        private void Button_OpenLong_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID)) return;

            var volume = GetVolume();
            if (volume < 1) return;

            var price = GetPrice();
            if (double.IsNaN(price)) return;

            var orderSn = ctpNet.OpenLong(instrumentID, volume, price);
            if (orderSn < 0)
            {
                Log("多头开仓发送失败:" + ((CtpNetResponseCode)orderSn).ToNameString());
                return;
            }

            Log("多头开仓发送成功" +
                ",报单序号=" + orderSn +
                ",instrumentID=" + instrumentID +
                ",volume=" + volume +
                ",price=" + price.ToString(CultureInfo.InvariantCulture));

            TextBox_OrderSn.Text = orderSn.ToString();
        }

        /// <summary>
        /// 多头平仓
        /// </summary>
        private void Button_CloseLong_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID)) return;

            var volume = GetVolume();
            if (volume < 1) return;

            var price = GetPrice();
            if (double.IsNaN(price)) return;

            var orderSn = ctpNet.CloseLong(instrumentID, volume, price);
            if (orderSn < 0)
            {
                Log("多头平仓发送失败:" + ((CtpNetResponseCode)orderSn).ToNameString());
                return;
            }

            if (orderSn == 0)
            {
                Log("多头平仓,上期所合约平仓数量中包含今仓和昨仓,指令被分拆为平今和平昨两条指令发送:" +
                    "instrumentID=" + instrumentID +
                    ",volume=" + volume +
                    ",price=" + price.ToString(CultureInfo.InvariantCulture));

                return;
            }

            Log("多头平仓发送成功," +
                "报单序号=" + orderSn +
                ",instrumentID=" + instrumentID +
                ",volume=" + volume +
                ",price=" + price.ToString(CultureInfo.InvariantCulture));

            TextBox_OrderSn.Text = orderSn.ToString();
        }

        /// <summary>
        /// 空头开仓
        /// </summary>
        private void Button_OpenShort_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID)) return;

            var volume = GetVolume();
            if (volume < 1) return;

            var price = GetPrice();
            if (double.IsNaN(price)) return;

            var orderSn = ctpNet.OpenShort(instrumentID, volume, price);
            if (orderSn < 0)
            {
                Log("空头开仓发送失败:" + ((CtpNetResponseCode)orderSn).ToNameString());
                return;
            }

            Log("空头开仓发送成功," +
                "报单序号=" + orderSn +
                ",instrumentID=" + instrumentID +
                ",volume=" + volume +
                ",price=" + price.ToString(CultureInfo.InvariantCulture));

            TextBox_OrderSn.Text = orderSn.ToString();
        }

        /// <summary>
        /// 空头平仓
        /// </summary>
        private void Button_CloseShort_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID)) return;

            var volume = GetVolume();
            if (volume < 1) return;

            var price = GetPrice();
            if (double.IsNaN(price)) return;

            var orderSn = ctpNet.CloseShort(instrumentID, volume, price);
            if (orderSn < 0)
            {
                Log("空头平仓发送失败:" + ((CtpNetResponseCode)orderSn).ToNameString());
                return;
            }

            if (orderSn == 0)
            {
                Log("空头平仓,上期所合约平仓数量中包含今仓和昨仓,指令被分拆为平今和平昨两条指令发送:" +
                    "instrumentID=" + instrumentID +
                    ",volume=" + volume +
                    ",price=" + price.ToString(CultureInfo.InvariantCulture));

                return;
            }

            Log("空头平仓发送成功," +
                "报单序号=" + orderSn +
                ",instrumentID=" + instrumentID +
                ",volume=" + volume +
                ",price=" + price.ToString(CultureInfo.InvariantCulture));

            TextBox_OrderSn.Text = orderSn.ToString();
        }

        /// <summary>
        /// 撤单
        /// </summary>
        private void Button_CancelOrder_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var orderSn = GetOrderSn();
            if (orderSn < 1) return;

            var responseCode = ctpNet.CancelOrder(orderSn);
            if (responseCode != CtpNetResponseCode.NONE)
            {
                Log("撤单发送失败:orderSn=" + orderSn + " :" + responseCode.ToNameString());
                return;
            }

            Log("撤单发送成功:orderSn=" + orderSn);
        }

        /// <summary>
        /// 批量撤单
        /// </summary>
        private void Button_CancelOrderEx_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            //合约代码
            var instrumentID = TextBox_InstrumentID.Text.Trim();

            //买卖方向
            var direction = GetDirection();

            //开平标志
            var offsetFlag = GetOffsetFlag();

            ctpNet.CancelOrderEx(instrumentID, direction, offsetFlag);
            Log("批量撤单:合约=" + instrumentID + " 买卖=" + direction.ToNameString() + " 开平=" + offsetFlag.ToNameString());
        }

        /// <summary>
        /// 自定义报单(套利单,条件单,FAK,FOK等)
        /// </summary>
        private void Button_InsertOrder_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID))
            {
                Log("请输入合约代码");
                return;
            }

            var direction = GetDirection();
            if (direction == 0)
            {
                Log("请选择买卖方向");
                return;
            }

            var offsetFlag = GetOffsetFlag();
            if (offsetFlag == 0)
            {
                Log("请选择开平标志");
                return;
            }

            var price = GetPrice();
            if (double.IsNaN(price)) return;

            var volume = GetVolume();
            if (volume < 1) return;

            //套利报单
            //instrumentID="SP a1701&a1705" '套利合约

            //市价单
            //条件单
            //....

            //FAK FOK
            //...

            var pRequest = new CtpNetInputOrderFieldEx();

            pRequest.CombHedgeFlag1 = CtpNetHedgeFlagType.Speculation;
            pRequest.CombOffsetFlag1 = offsetFlag;
            pRequest.ContingentCondition = CtpNetContingentConditionType.Immediately;
            pRequest.Direction = direction;
            pRequest.InstrumentID = instrumentID;
            pRequest.LimitPrice = price;
            pRequest.MinVolume = 1;
            pRequest.OrderPriceType = CtpNetOrderPriceTypeType.LimitPrice;
            pRequest.StopPrice = 0;
            pRequest.TimeCondition = CtpNetTimeConditionType.GFD;
            pRequest.VolumeCondition = CtpNetVolumeConditionType.AV;
            pRequest.VolumeTotalOriginal = volume;

            var orderSn = ctpNet.InsertOrder(pRequest);
            if (orderSn < 0)
            {
                Log("报单发送失败:" + ((CtpNetResponseCode)orderSn).ToNameString());
                return;
            }

            Log("报单发送成功," +
                "报单序号=" + orderSn +
                ",instrumentID=" + instrumentID +
                ",volume=" + volume +
                ",price=" + price.ToString(CultureInfo.InvariantCulture));

            TextBox_OrderSn.Text = orderSn.ToString();
        }

        /// <summary>
        /// 报单是否可撤
        /// </summary>
        private void Button_IsOrderOpen_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var orderSn = GetOrderSn();
            if (orderSn < 1) return;

            var yesNo = ctpNet.IsOrderOpen(orderSn);

            if (yesNo)
            {
                Log("报单序号=" + orderSn + "的报单还在挂单");
            }
            else
            {
                Log("报单序号=" + orderSn + "的报单已不在挂单");
            }
        }

        /// <summary>
        /// 获取未成交报单
        /// </summary>
        private void Button_GetTradingOrderSnArray_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            //合约代码,空字符串为不指定
            var instrumentID = TextBox_InstrumentID.Text.Trim();

            //买卖方向
            var direction = GetDirection();

            //开平标志
            var offsetFlag = GetOffsetFlag();

            //报单状态
            var orderStatus = (CtpNetOrderStatusType)1; //特殊值:1 = 还在挂单

            var orderSnList = ctpNet.GetOrderSnList(instrumentID, direction, offsetFlag, orderStatus);

            if (orderSnList.Count < 1)
            {
                Log("没有未成交报单");
                return;
            }

            Log("未成交报单数量=" + orderSnList.Count);
            foreach (var sn in orderSnList)
            {
                Log("报单序号=" + sn);
            }
        }

        /// <summary>
        /// 获取报单信息
        /// </summary>
        private void Button_GetOrderInfoByOrderSn_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var orderSn = GetOrderSn();
            if (orderSn < 1) return;

            var order = ctpNet.GetOrder(orderSn);
            if (order == null)
            {
                Log("报单序号=" + orderSn + "的报单信息获取失败");
            }
            else
            {
                Log("报单序号=" + orderSn + "的报单信息:" +
                    "instrumentID=" + order.InstrumentID +
                    ",volumeTotal=" + order.VolumeTotal +
                    ",price=" + order.LimitPrice.ToString(CultureInfo.InvariantCulture));
            }
        }

        /// <summary>
        /// 检索所有报单记录
        /// </summary>
        private void Button_GetOrderInfoByIndex_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            //不指定所有条件,即为获取所有报单的报单序号
            var orderSnList = ctpNet.GetOrderSnList();
            if (orderSnList == null)
            {
                Log("检索所有报单记录失败");
                return;
            }

            if (orderSnList.Count < 1)
            {
                Log("没有报单记录");
                return;
            }

            Log("报单记录数量=" + orderSnList.Count);
            foreach (var sn in orderSnList)
            {
                var order = ctpNet.GetOrder(sn);
                Log("报单序号=" + sn + "的报单信息:" +
                        "instrumentID=" + order.InstrumentID +
                        ",volumeTotal=" + order.VolumeTotal +
                        ",price=" + order.LimitPrice.ToString(CultureInfo.InvariantCulture));
            }
        }

        /// <summary>
        /// 检索所有成交记录
        /// </summary>
        private void Button_GetTradeRecordByIndex_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var tradeList = ctpNet.GetTrades();

            Log("成交记录数量=" + tradeList.Count);
            foreach (var trade in tradeList)
            {
                Log("成交信息:" +
                    "instrumentID=" + trade.InstrumentID +
                    ",orderSn=" + trade.OrderSn +
                    ",price=" + trade.Price.ToString(CultureInfo.InvariantCulture));
            }
        }

        /// <summary>
        /// 获取当前持仓
        /// </summary>
        private void Button_GetPositionInfoByIndex_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var positionList = ctpNet.GetPositions();

            Log("持仓数量=" + positionList.Count);
            foreach (var pi in positionList)
            {
                Log("持仓信息:" +
                           "instrumentID=" + pi.InstrumentID +
                           ",direction=" + pi.Direction.ToNameString() +
                           ",position=" + pi.Position);
            }
        }

        /// <summary>
        /// 获取指定合约持仓
        /// </summary>
        private void Button_GetPositionInfoById_Click(object sender, EventArgs e)
        {
            if (CheckConnection() == false) return;

            var instrumentID = GetInstrumentID();
            if (string.IsNullOrWhiteSpace(instrumentID))
            {
                Log("请输入合约代码");
                return;
            }

            var direction = GetDirection();
            if (direction == 0)
            {
                Log("请选择买卖方向");
                return;
            }

            //投机套保标志类型
            var hedgeFlag = CtpNetHedgeFlagType.Speculation;

            var strDirection = direction == CtpNetDirectionType.Buy ? "多头持仓" : "空头持仓";

            var pi = ctpNet.GetPosition(instrumentID, direction, hedgeFlag);
            if (pi == null)
            {
                Log(instrumentID + strDirection + " 获取指定合约持仓失败");
            }
            else
            {
                Log(instrumentID + strDirection + " 持仓=" + pi.Position);
            }
        }
        
        #endregion

        #region 接口事件

        /// <summary>
        /// 日志
        /// </summary>
        /// <param name="pLogInfo"></param>
        private void CtpNet_OnLog(CtpNetLogInfoFieldEx pLogInfo)
        {
            if (CheckBox_ShowOnLog.Checked == false) return;

            LogEvent("[日志]" + pLogInfo.LogContent);
        }

        /// <summary>
        /// 连接到行情服务器
        /// </summary>
        private void CtpNet_OnMarketDataServerConnected()
        {
            LogEvent("连接到行情服务器");
        }

        /// <summary>
        /// 行情服务器断开
        /// </summary>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnMarketDataServerDisconnected(CtpNetRspInfoField pRspInfo)
        {
            LogEvent("行情服务器断开");
        }

        /// <summary>
        /// 连接到交易服务器
        /// </summary>
        private void CtpNet_OnTradeServerConnected()
        {
            LogEvent("连接到交易服务器");
        }

        /// <summary>
        /// 交易服务器断开
        /// </summary>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnTradeServerDisconnected(CtpNetRspInfoField pRspInfo)
        {
            LogEvent("交易服务器断开");
        }

        /// <summary>
        /// 帐号初始化响应
        /// </summary>
        private void CtpNetOnInitialized()
        {
            LogEvent("初始化完成");
        }

        /// <summary>
        /// CtpNet清理响应
        /// </summary>
        /// <param name="pRspInfo"></param>
        private void OnReleased(CtpNetRspInfoField pRspInfo)
        {
            LogEvent("清理响应:" + pRspInfo.ResponseMsg());
        }

        /// <summary>
        /// 行情通知
        /// </summary>
        /// <param name="pMarketData"></param>
        private void CtpNet_OnMarketData(CtpNetDepthMarketDataField pMarketData)
        {
            if (CheckBox_ShowOnMarketData.Checked == false) return;

            LogEvent("行情通知:InstrumentID=" + pMarketData.InstrumentID + ",LastPrice=" + pMarketData.LastPrice);
        }

        /// <summary>
        /// 合约状态通知
        /// </summary>
        /// <param name="pInstrumentStatus"></param>
        private void CtpNet_OnInstrumentStatus(CtpNetInstrumentStatusField pInstrumentStatus)
        {
            if (CheckBox_ShowOnInstrumentStatus.Checked == false) return;

            LogEvent("合约状态通知:" + pInstrumentStatus.InstrumentID + " " + pInstrumentStatus.InstrumentStatus.ToNameString());
        }

        /// <summary>
        /// 资金变化通知
        /// </summary>
        /// <param name="pTradingAccount"></param>
        private void CtpNet_OnTradingAccount(CtpNetTradingAccountField pTradingAccount)
        {
            if (CheckBox_ShowOnAccountMoney.Checked == false) return;

            LogEvent("资金变化通知:动态权益=" + pTradingAccount.Balance + ",持仓盈亏=" + pTradingAccount.PositionProfit);
        }

        /// <summary>
        /// 交易通知
        /// </summary>
        /// <param name="pNoticeInfo"></param>
        private void CtpNet_OnTradingNotice(CtpNetTradingNoticeInfoField pNoticeInfo)
        {
            if (pNoticeInfo == null) return;
            LogEvent("交易通知:" + pNoticeInfo.FieldContent);
        }

        /// <summary>
        /// 查询银行余额响应
        /// </summary>
        /// <param name="pBankAccountInfo"></param>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnQueryBankAccountMoney(CtpNetNotifyQueryAccountField pBankAccountInfo, CtpNetRspInfoField pRspInfo)
        {
            if (pRspInfo.IsError())
            {
                LogEvent("查询银行余额失败:" + pRspInfo.ResponseMsg());
                return;
            }

            LogEvent("银行余额:银行可用金额=" + pBankAccountInfo.BankUseAmount + ",银行可取金额=" + pBankAccountInfo.BankFetchAmount);
        }

        /// <summary>
        /// 银行资金转期货响应(入金)
        /// </summary>
        /// <param name="pTransferInfo"></param>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnFromBankToFuture(CtpNetRspTransferField pTransferInfo, CtpNetRspInfoField pRspInfo)
        {
            if (pRspInfo.IsError())
            {
                LogEvent("银行资金转期货失败:" + pRspInfo.ResponseMsg());
                return;
            }

            LogEvent("银行资金转期货成功,转帐金额=" + pTransferInfo.TradeAmount);
        }

        /// <summary>
        /// 期货资金转银行响应(出金)
        /// </summary>
        /// <param name="pTransferInfo"></param>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnFromFutureToBank(CtpNetRspTransferField pTransferInfo, CtpNetRspInfoField pRspInfo)
        {
            if (pRspInfo.IsError())
            {
                LogEvent("期货资金转银行失败:" + pRspInfo.ResponseMsg());
                return;
            }

            LogEvent("期货资金转银行成功,转帐金额=" + pTransferInfo.TradeAmount);
        }

        /// <summary>
        /// 报单通知
        /// </summary>
        /// <param name="pOrder"></param>
        private void CtpNet_OnOrder(CtpNetOrderFieldEx pOrder)
        {
            //报单失败
            if (pOrder.OrderSubmitStatus == CtpNetOrderSubmitStatusType.InsertRejected)
            {
                if (CheckBox_ShowOnOrderFailed.Checked == false) return;
                LogEvent("报单失败:报单序号=" + pOrder.OrderSn + " " + pOrder.StatusMsg);

                return;
            }

            //撤单失败
            if (pOrder.OrderSubmitStatus == CtpNetOrderSubmitStatusType.ModifyRejected)
            {
                if (CheckBox_ShowOnCancelOrderFailed.Checked == false) return;
                LogEvent("撤单失败:报单序号=" + pOrder.OrderSn + " " + pOrder.StatusMsg);

                return;
            }

            if (CheckBox_ShowOnOrder.Checked == false) return;

            LogEvent("报单通知:报单序号=" + pOrder.OrderSn + " " + pOrder.StatusMsg);
        }

        /// <summary>
        /// 成交通知
        /// </summary>
        /// <param name="pTrade"></param>
        private void CtpNet_OnTrade(CtpNetTradeFieldEx pTrade)
        {
            if (CheckBox_ShowOnTrade.Checked == false) return;

            LogEvent("成交通知:报单序号=" + pTrade.OrderSn + " " + pTrade.Direction.ToNameString() + " " +
                     pTrade.OffsetFlag.ToNameString() + " " + pTrade.Volume + "手,价格=" + pTrade.Price);
        }

        /// <summary>
        /// 持仓变化通知
        /// </summary>
        /// <param name="pPosition"></param>
        private void CtpNet_OnPosition(CtpNetInvestorPositionFieldEx pPosition)
        {
            if (CheckBox_ShowOnPosition.Checked == false) return;

            LogEvent("持仓变化通知:" + pPosition.InstrumentID + " " +
                     pPosition.Direction.ToNameString() + " " +
                     pPosition.HedgeFlag.ToNameString() + " 共" + pPosition.Position + "手,可平=" +
                     pPosition.ClosablePosition);
        }

        /// <summary>
        /// 查询手续费率响应
        /// </summary>
        /// <param name="pCommissionRate"></param>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnQueryCommissionRate(CtpNetInstrumentCommissionRateField pCommissionRate, CtpNetRspInfoField pRspInfo)
        {
            if (pRspInfo.IsError())
            {
                LogEvent("查询手续费率失败:" + pRspInfo.ResponseMsg());
                return;
            }

            if (pCommissionRate == null)
                return;

            LogEvent("查询手续费率成功," +
                     "InstrumentID=" + pCommissionRate.InstrumentID +
                     ",OpenRatioByMoney=" + pCommissionRate.OpenRatioByMoney +
                     ",OpenRatioByVolume=" + pCommissionRate.OpenRatioByVolume);
        }

        /// <summary>
        /// 查询保证金率响应
        /// </summary>
        /// <param name="pMarginRate"></param>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnQueryMarginRate(CtpNetInstrumentMarginRateField pMarginRate, CtpNetRspInfoField pRspInfo)
        {
            if (pRspInfo.IsError())
            {
                LogEvent("查询保证金率失败:" + pRspInfo.ResponseMsg());
                return;
            }

            if (pMarginRate == null)
                return;

            LogEvent("查询保证金率成功," +
                     "InstrumentID=" + pMarginRate.InstrumentID +
                     ",LongMarginRatioByMoney=" + pMarginRate.LongMarginRatioByMoney +
                     ",LongMarginRatioByVolume=" + pMarginRate.LongMarginRatioByVolume);
        }

        /// <summary>
        /// 查询报单手续费率响应
        /// </summary>
        /// <param name="pOrderCommissionRate"></param>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnQueryOrderCommissionRate(CtpNetInstrumentOrderCommRateField pOrderCommissionRate, CtpNetRspInfoField pRspInfo)
        {
            if (pRspInfo.IsError())
            {
                LogEvent("查询报单手续费率失败:" + pRspInfo.ResponseMsg());
                return;
            }

            if (pOrderCommissionRate == null)
                return;

            LogEvent("查询报单手续费率成功," +
                     "InstrumentID=" + pOrderCommissionRate.InstrumentID +
                     ",OrderCommByVolume=" + pOrderCommissionRate.OrderCommByVolume +
                     ",OrderActionCommByVolume=" + pOrderCommissionRate.OrderActionCommByVolume);
        }

        /// <summary>
        /// 更新交易密码响应
        /// </summary>
        /// <param name="pPasswordInfo"></param>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnUpdateUserPassword(CtpNetUserPasswordUpdateField pPasswordInfo, CtpNetRspInfoField pRspInfo)
        {
            if (pRspInfo.IsError())
            {
                LogEvent("更新交易密码失败:" + pRspInfo.ResponseMsg());
                return;
            }

            LogEvent("更新交易密码成功");
        }

        /// <summary>
        /// 更新资金密码响应
        /// </summary>
        /// <param name="pPasswordInfo"></param>
        /// <param name="pRspInfo"></param>
        private void CtpNet_OnUpdateTradingAccountPassword(CtpNetTradingAccountPasswordUpdateField pPasswordInfo, CtpNetRspInfoField pRspInfo)
        {
            if (pRspInfo.IsError())
            {
                LogEvent("更新资金密码失败:" + pRspInfo.ResponseMsg());
                return;
            }

            LogEvent("更新资金密码成功");
        }
        
        #endregion
    }
}
