﻿Imports TaiTanXing.Ctp.CtpNet

Public Class MainForm

    ''' <summary>
    ''' CtpNet组件
    ''' </summary>
    Private WithEvents ctpNet As CtpNetClass

#Region "辅助函数"

    ''' <summary>
    ''' 清空函数日志
    ''' </summary>
    Private Sub Button_ClearFunctionLog_Click(sender As Object, e As EventArgs) Handles Button_ClearFunctionLog.Click
        TextBox_Log.Clear()
    End Sub

    ''' <summary>
    ''' 清空事件日志
    ''' </summary>
    Private Sub Button_ClearEventLog_Click(sender As Object, e As EventArgs) Handles Button_ClearEventLog.Click
        TextBox_EventLog.Clear()
    End Sub

    ''' <summary>
    ''' 窗体加载
    ''' </summary>
    Private Sub MainForm_Load(sender As Object, e As EventArgs) Handles MyBase.Load

    End Sub
    
    ''' <summary>
    ''' 窗体已关闭
    ''' </summary>
    Private Sub MainForm_FormClosed(sender As Object, e As FormClosedEventArgs) Handles Me.FormClosed
        '清理组件
        If IsNothing(ctpNet) = False Then
            Try
                ctpNet.Release()
            Catch ex As Exception

            End Try
        End If

        Environment.Exit(0)
    End Sub

    ''' <summary>
    ''' 检查是否已登录
    ''' </summary>
    Private Function CheckConnection() As Boolean
        If IsNothing(ctpNet) OrElse ctpNet.IsTradeServerLogined = False Then
            Log("请先登录")
            Return False
        End If

        Return True
    End Function

    ''' <summary>
    ''' 获取窗体界面上输入的合约代码
    ''' </summary>
    Private Function GetInstrumentID() As String
        Return TextBox_InstrumentID.Text.Trim()
    End Function

    ''' <summary>
    ''' 获取窗体界面上输入的报单数量,失败返回 -1
    ''' </summary>
    ''' <returns></returns>
    Private Function GetVolume() As Integer
        Dim str = TextBox_Volume.Text.Trim()
        If String.IsNullOrWhiteSpace(str) Then
            Log("请输入报单数量")
            TextBox_Volume.Clear()
            TextBox_Volume.Focus()
            Return -1
        End If

        Dim intVolume As Integer
        If Integer.TryParse(str, intVolume) = False Or intVolume < 1 Then
            Log("输入的报单数量无效:" & str)
            TextBox_Volume.Clear()
            TextBox_Volume.Focus()
            Return -1
        End If

        Return intVolume
    End Function

    ''' <summary>
    ''' 获取窗体界面上输入的报单价格,失败返回 Double.NaN
    ''' </summary>
    ''' <returns></returns>
    Private Function GetPrice() As Double
        Dim str = TextBox_Price.Text.Trim()
        If String.IsNullOrWhiteSpace(str) Then
            Log("请输入报单价格")
            TextBox_Price.Clear()
            TextBox_Price.Focus()
            Return Double.NaN
        End If

        Dim dblPrice As Double
        If Double.TryParse(str, dblPrice) = False Then
            Log("输入的报单价格无效:" & str)
            TextBox_Price.Clear()
            TextBox_Price.Focus()
            Return Double.NaN
        End If

        Return dblPrice
    End Function

    ''' <summary>
    ''' 获取窗体界面上输入的持仓方向
    ''' </summary>
    ''' <returns></returns>
    Private Function GetDirection() As CtpNetDirectionType

        If radioButton_Buy.Checked Then
            Return CtpNetDirectionType.Buy
        End If

        If radioButton_Sell.Checked Then
            Return CtpNetDirectionType.Sell
        End If

        Return 0 '不指定

    End Function

    ''' <summary>
    ''' 获取窗体界面上输入的开平标志
    ''' </summary>
    ''' <returns></returns>
    Private Function GetOffsetFlag() As CtpNetOffsetFlagType

        If radioButton_Open.Checked Then
            Return CtpNetOffsetFlagType.Open
        End If

        If radioButton_Close.Checked Then
            Return CtpNetOffsetFlagType.Close
        End If

        If radioButton_CloseYesterday.Checked Then
            Return CtpNetOffsetFlagType.CloseYesterday
        End If

        If radioButton_CloseToday.Checked Then
            Return CtpNetOffsetFlagType.CloseToday
        End If

        Return 0 '不指定

    End Function

    ''' <summary>
    ''' 获取窗体界面上输入的报单序号,失败返回 -1
    ''' </summary>
    ''' <returns></returns>
    Private Function GetOrderSn() As Integer
        Dim str = TextBox_OrderSn.Text.Trim()
        If String.IsNullOrWhiteSpace(str) Then
            Log("请输入报单序号")
            TextBox_OrderSn.Clear()
            TextBox_OrderSn.Focus()
            Return -1
        End If

        Dim intOrderSn As Integer
        If Integer.TryParse(str, intOrderSn) = False Or intOrderSn < 1 Then
            Log("输入的报单序号无效:" & str)
            TextBox_OrderSn.Clear()
            TextBox_OrderSn.Focus()
            Return -1
        End If

        Return intOrderSn
    End Function

    ''' <summary>
    ''' 添加日志
    ''' </summary>
    ''' <param name="txt">日志内容</param>
    Private Sub Log(txt As String)
        If String.IsNullOrWhiteSpace(txt) Then Exit Sub
        If IsDisposed Or Disposing Then Exit Sub

        Try
            If InvokeRequired Then
                Dim a = New Action(Of String)(AddressOf Log)
                Invoke(a, txt)
                Exit Sub
            End If

            txt = txt.Trim()
            TextBox_Log.AppendText(String.Format("{0} {1}{2}", Date.Now.ToString("yyyy-MM-dd HH:mm:ss.fff"), txt, Environment.NewLine))
        Catch ex As Exception

        End Try
    End Sub

    ''' <summary>
    ''' 添加事件日志
    ''' </summary>
    ''' <param name="txt">日志内容</param>
    Private Sub LogEvent(txt As String)
        If String.IsNullOrWhiteSpace(txt) Then Exit Sub
        If IsDisposed Or Disposing Then Exit Sub

        If InvokeRequired Then
            Try
                Dim a = New Action(Of String)(AddressOf LogEvent)
                Invoke(a, txt)
                Exit Sub
            Catch ex As Exception

            End Try
        End If

        Try
            txt = txt.Trim()
            TextBox_EventLog.AppendText(String.Format("{0} {1}{2}", Date.Now.ToString("yyyy-MM-dd HH:mm:ss.fff"), txt, Environment.NewLine))
        Catch ex As Exception

        End Try
    End Sub

#End Region

#Region "调用接口功能"

    ''' <summary>
    ''' 登录
    ''' </summary>
    Private Sub Button_Login_Click(sender As Object, e As EventArgs) Handles Button_Login.Click
        
        Log("发送登录请求")

        '清理旧组件
        If IsNothing(ctpNet) = False Then
            ctpNet.Release()
            ctpNet = Nothing
        End If

        '创建组件实例
        ctpNet = New CtpNetClass()

        Dim loginInfo As New CtpNetReqUserLoginFieldEx()

        'Dim responseCode = loginInfo.ReadConfigFile("快期的配置文件路径", "服务器名称") '读取快期的配置文件
        'If responseCode <> CtpNetResponseCode.NONE Then
        '    Log("读取快期的配置文件失败:" & responseCode.ToNameString())
        '    Exit Sub
        'End If

        loginInfo.UserID = "034180" 'TODO:
        loginInfo.Password = "123" 'TODO:
        loginInfo.BrokerID = "9999" 'TODO:

        loginInfo.MarketDataServerAddress.Add("180.168.146.187:10031") 'TODO:
        loginInfo.TradeServerAddress.Add("180.168.146.187:10030") 'TODO:
        '
        'loginInfo.MarketDataServerAddress.Add("180.168.146.187:10010") '添加其它服务器地址
        'loginInfo.TradeServerAddress.Add("180.168.146.187:10000") '添加其它服务器地址
        
        loginInfo.AuthCode = ""
        loginInfo.UserProductInfo = ""
        '等待初始化完成的时间(秒), 小于1秒,表示以异步方式登录,函数立即返回,可以在初始化完成事件中得到结果; 
        '大于等于1秒,表示以同步方式登录,函数直到初始化完成或者超时才返回
        'loginInfo.WaitTime = 30 

        Dim responseCode = ctpNet.Login(loginInfo)
        If responseCode <> CtpNetResponseCode.NONE Then
            Log("发送登录请求失败:" & responseCode.ToNameString())
            Exit Sub
        End If

        Log("发送登录请求成功")
    End Sub
    
    ''' <summary>
    ''' 获取交易日
    ''' </summary>
    Private Sub Button_TradingDay_Click(sender As Object, e As EventArgs) Handles Button_TradingDay.Click
        If CheckConnection() = False Then Exit Sub
        Log("交易日:" & ctpNet.TradingDay.ToString())
    End Sub

    ''' <summary>
    ''' 获取合约信息
    ''' </summary>
    Private Sub Button_Instruments_Click(sender As Object, e As EventArgs) Handles Button_Instruments.Click
        If CheckConnection() = False Then Exit Sub

        Dim dic = ctpNet.GetInstruments()
        Log("合约数量:" & dic.Count)

        For Each instrument In dic.Values
            Log(instrument.InstrumentID & " " & instrument.InstrumentName)
        Next
    End Sub

    ''' <summary>
    ''' 获取合约状态
    ''' </summary>
    Private Sub Button_GetInstrumentStatus_Click(sender As Object, e As EventArgs) Handles Button_GetInstrumentStatus.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then Exit Sub

        Dim status = ctpNet.GetInstrumentStatus(instrumentID)
        If IsNothing(status) Then
            Log("获取 " & instrumentID & " 的状态失败")
            Exit Sub
        End If

        Log("合约状态 " & instrumentID & " " & status.InstrumentStatus.ToNameString())
    End Sub

    ''' <summary>
    ''' 订阅行情
    ''' </summary>
    Private Sub Button_SubscribeMarketData_Click(sender As Object, e As EventArgs) Handles Button_SubscribeMarketData.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then Exit Sub

        Dim responseCode = ctpNet.SubscribeMarketData(instrumentID)
        If responseCode <> CtpNetResponseCode.NONE Then
            Log("订阅 " & instrumentID & " 的行情发送失败:" & responseCode.ToNameString())
            Exit Sub
        End If

        Log("订阅 " & instrumentID & " 的行情发送成功")
    End Sub

    ''' <summary>
    ''' 获取行情
    ''' </summary>
    Private Sub Button_GetMarketData_Click(sender As Object, e As EventArgs) Handles Button_GetMarketData.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then Exit Sub

        Dim tick = ctpNet.GetMarketData(instrumentID)

        If IsNothing(tick) Then
            Log("获取 " & instrumentID & " 的行情失败")
            Exit Sub
        End If

        Log(instrumentID & " 的详细行情:最新价=" & tick.LastPrice.ToString() &
            ",卖一价=" & tick.AskPrice1.ToString() &
            ",买一价=" & tick.BidPrice1.ToString() &
            ",行情时间=" & tick.UpdateTime)
    End Sub

    ''' <summary>
    ''' 获取帐号资金
    ''' </summary>
    Private Sub Button_TradingAccount_Click(sender As Object, e As EventArgs) Handles Button_TradingAccount.Click
        If CheckConnection() = False Then Exit Sub

        Log("动态权益=" & ctpNet.TradingAccount.Balance.ToString() & ",可用资金=" & ctpNet.TradingAccount.Available.ToString())
    End Sub

    ''' <summary>
    ''' 多头开仓
    ''' </summary>
    Private Sub Button_OpenLong_Click(sender As Object, e As EventArgs) Handles Button_OpenLong.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then Exit Sub

        Dim volume = GetVolume()
        If volume < 1 Then Exit Sub

        Dim price = GetPrice()
        If Double.IsNaN(price) Then Exit Sub

        Dim orderSn = ctpNet.OpenLong(instrumentID, volume, price)
        If orderSn < 0 Then
            Log("多头开仓发送失败:instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString() & " :" & CType(orderSn, CtpNetResponseCode).ToNameString())
            Exit Sub
        End If

        Log("多头开仓发送成功,报单序号=" & orderSn.ToString() & ",instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString())
        TextBox_OrderSn.Text = orderSn.ToString()
    End Sub

    ''' <summary>
    ''' 多头平仓
    ''' </summary>
    Private Sub Button_CloseLong_Click(sender As Object, e As EventArgs) Handles Button_CloseLong.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then Exit Sub

        Dim volume = GetVolume()
        If volume < 1 Then Exit Sub

        Dim price = GetPrice()
        If Double.IsNaN(price) Then Exit Sub

        Dim orderSn = ctpNet.CloseLong(instrumentID, volume, price)
        If orderSn < 0 Then
            Log("多头平仓发送失败:instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString() & " :" & CType(orderSn, CtpNetResponseCode).ToNameString())
            Exit Sub
        End If

        If orderSn = 0 Then
            Log("多头平仓,上期所合约平仓数量中包含今仓和昨仓,指令被分拆为平今和平昨两条指令发送:instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString())
            Exit Sub
        End If

        Log("多头平仓发送成功,报单序号=" & orderSn.ToString() & ",instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString())
        TextBox_OrderSn.Text = orderSn.ToString()
    End Sub

    ''' <summary>
    ''' 空头开仓
    ''' </summary>
    Private Sub Button_OpenShort_Click(sender As Object, e As EventArgs) Handles Button_OpenShort.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then Exit Sub

        Dim volume = GetVolume()
        If volume < 1 Then Exit Sub

        Dim price = GetPrice()
        If Double.IsNaN(price) Then Exit Sub

        Dim orderSn = ctpNet.OpenShort(instrumentID, volume, price)
        If orderSn < 0 Then
            Log("空头开仓发送失败:instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString() & " :" & CType(orderSn, CtpNetResponseCode).ToNameString())
            Exit Sub
        End If

        Log("空头开仓发送成功,报单序号=" & orderSn.ToString() & ",instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString())
        TextBox_OrderSn.Text = orderSn.ToString()
    End Sub

    ''' <summary>
    ''' 空头平仓
    ''' </summary>
    Private Sub Button_CloseShort_Click(sender As Object, e As EventArgs) Handles Button_CloseShort.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then Exit Sub

        Dim volume = GetVolume()
        If volume < 1 Then Exit Sub

        Dim price = GetPrice()
        If Double.IsNaN(price) Then Exit Sub

        Dim orderSn = ctpNet.CloseShort(instrumentID, volume, price)
        If orderSn < 0 Then
            Log("空头平仓发送失败:instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString() & " :" & CType(orderSn, CtpNetResponseCode).ToNameString())
            Exit Sub
        End If

        If orderSn = 0 Then
            Log("空头平仓,上期所合约平仓数量中包含今仓和昨仓,指令被分拆为平今和平昨两条指令发送:instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString())
            Exit Sub
        End If

        Log("空头平仓发送成功,报单序号=" & orderSn.ToString() & ",instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString())
        TextBox_OrderSn.Text = orderSn.ToString()
    End Sub

    ''' <summary>
    ''' 撤单
    ''' </summary>
    Private Sub Button_CancelOrder_Click(sender As Object, e As EventArgs) Handles Button_CancelOrder.Click
        If CheckConnection() = False Then Exit Sub

        Dim orderSn = GetOrderSn()
        If orderSn < 1 Then Exit Sub

        Dim responseCode = ctpNet.CancelOrder(orderSn)
        If responseCode <> CtpNetResponseCode.NONE Then
            Log("撤单发送失败:orderSn=" & orderSn.ToString() & " :" & responseCode.ToNameString())
            Exit Sub
        End If

        Log("撤单发送成功:orderSn=" & orderSn.ToString())
    End Sub

    ''' <summary>
    ''' 批量撤单
    ''' </summary>
    Private Sub Button_CancelOrderEx_Click(sender As Object, e As EventArgs) Handles Button_CancelOrderEx.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = TextBox_InstrumentID.Text.Trim()
        Dim direction = GetDirection()
        Dim offsetFlag = GetOffsetFlag()

        ctpNet.CancelOrderEx(instrumentID, direction, offsetFlag)
        Log("批量撤单:合约=" & instrumentID & " 买卖=" & direction.ToNameString() & " 开平=" & offsetFlag.ToNameString())
    End Sub

    ''' <summary>
    ''' 自定义报单(套利单,条件单,FAK,FOK等)
    ''' </summary>
    Private Sub Button_InsertOrder_Click(sender As Object, e As EventArgs) Handles Button_InsertOrder.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then
            Log("请输入合约代码")
            Exit Sub
        End If

        Dim direction = GetDirection()
        If direction = 0 Then
            Log("请选择买卖方向")
            Exit Sub
        End If

        Dim offsetFlag = GetOffsetFlag()
        If offsetFlag = 0 Then
            Log("请选择开平标志")
            Exit Sub
        End If

        Dim price = GetPrice()
        If Double.IsNaN(price) Then Exit Sub

        Dim volume = GetVolume()
        If volume < 1 Then Exit Sub

        '套利报单
        'instrumentID="SP a1701&a1705" '套利合约

        '市价单
        'orderPriceType=49 '任意价(AnyPrice)
        'price=0           '
        'timeCondition=49   '立即完成,否则撤销(IOC)

        '条件单
        '....

        'FAK FOK
        '...

        Dim pRequest As New CtpNetInputOrderFieldEx

        pRequest.CombHedgeFlag1 = CtpNetHedgeFlagType.Speculation
        pRequest.CombOffsetFlag1 = offsetFlag
        pRequest.ContingentCondition = CtpNetContingentConditionType.Immediately
        pRequest.Direction = direction
        pRequest.InstrumentID = instrumentID
        pRequest.LimitPrice = price
        pRequest.MinVolume = 1
        pRequest.OrderPriceType = CtpNetOrderPriceTypeType.LimitPrice
        pRequest.StopPrice = 0
        pRequest.TimeCondition = CtpNetTimeConditionType.GFD
        pRequest.VolumeCondition = CtpNetVolumeConditionType.AV
        pRequest.VolumeTotalOriginal = volume

        Dim orderSn = ctpNet.InsertOrder(pRequest)
        If orderSn < 0 Then
            Log("报单发送失败:instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString() & " :" & CType(orderSn, CtpNetResponseCode).ToNameString())
            Exit Sub
        End If

        Log("报单发送成功,报单序号=" & orderSn.ToString() & ",instrumentID=" & instrumentID & ",volume=" & volume.ToString() & ",price=" & price.ToString())
        TextBox_OrderSn.Text = orderSn.ToString()

    End Sub

    ''' <summary>
    ''' 报单是否可撤
    ''' </summary>
    Private Sub Button_IsOrderOpen_Click(sender As Object, e As EventArgs) Handles Button_IsOrderOpen.Click
        If CheckConnection() = False Then Exit Sub

        Dim orderSn = GetOrderSn()
        If orderSn < 1 Then Exit Sub

        Dim yesNo = ctpNet.IsOrderOpen(orderSn)
        If yesNo Then
            Log("报单序号=" & orderSn.ToString() & "的报单还在挂单")
        Else
            Log("报单序号=" & orderSn.ToString() & "的报单已不在挂单")
        End If
    End Sub

    ''' <summary>
    ''' 获取未成交报单
    ''' </summary>
    Private Sub Button_GetTradingOrderSnList_Click(sender As Object, e As EventArgs) Handles Button_GetTradingOrderSnList.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = TextBox_InstrumentID.Text.Trim()
        Dim direction = GetDirection()
        Dim offsetFlag = GetOffsetFlag()
        Dim orderStatus As CtpNetOrderStatusType = 1 '特殊值:1 = 还在挂单

        Dim orderSnList = ctpNet.GetOrderSnList(instrumentID, direction, offsetFlag, orderStatus)

        If orderSnList.Count < 1 Then
            Log("没有未成交报单")
            Exit Sub
        End If

        Log("未成交报单数量=" & orderSnList.Count.ToString())

        For Each orderSn In orderSnList
            Log("未成交报单序号=" & orderSn.ToString())
        Next

    End Sub

    ''' <summary>
    ''' 获取报单信息
    ''' </summary>
    Private Sub Button_GetOrder_Click(sender As Object, e As EventArgs) Handles Button_GetOrder.Click
        If CheckConnection() = False Then Exit Sub

        Dim orderSn = GetOrderSn()
        If orderSn < 1 Then Exit Sub

        Dim order = ctpNet.GetOrder(orderSn)
        If IsNothing(order) Then
            Log("报单序号=" & orderSn.ToString() & "的报单信息获取失败")
        Else
            Log("报单序号=" & orderSn.ToString() & "的报单信息:instrumentID=" & order.InstrumentID & ",volumeTotal=" & order.VolumeTotal.ToString() & ",price=" & order.LimitPrice.ToString())
        End If
    End Sub

    ''' <summary>
    ''' 检索所有报单记录
    ''' </summary>
    Private Sub Button_GetOrders_Click(sender As Object, e As EventArgs) Handles Button_GetOrders.Click
        If CheckConnection() = False Then Exit Sub

        Dim orderSnList = ctpNet.GetOrderSnList() '不指定条件,即为获取所有报单的报单序号

        If orderSnList.Count < 1 Then
            Log("没有报单记录")
            Exit Sub
        End If

        Log("报单记录数量=" & orderSnList.Count.ToString())

        For Each sn As Integer In orderSnList
            Dim order = ctpNet.GetOrder(sn)
            Log("报单序号=" & sn.ToString() & "的报单信息:instrumentID=" & order.InstrumentID & ",volumeTotal=" & order.VolumeTotal.ToString() & ",price=" & order.LimitPrice.ToString())
        Next

    End Sub

    ''' <summary>
    ''' 检索所有成交记录
    ''' </summary>
    Private Sub Button_Trades_Click(sender As Object, e As EventArgs) Handles Button_Trades.Click
        If CheckConnection() = False Then Exit Sub

        Dim tradeList = ctpNet.GetTrades()
        Log("成交记录数量=" & tradeList.Count.ToString())

        For Each trade In tradeList
            Log("成交信息:instrumentID=" & trade.InstrumentID & ",orderSn=" & trade.OrderSn.ToString() & ",price=" & trade.Price.ToString())
        Next
    End Sub

    ''' <summary>
    ''' 获取当前持仓
    ''' </summary>
    Private Sub Button_GetPositions_Click(sender As Object, e As EventArgs) Handles Button_GetPositions.Click
        If CheckConnection() = False Then Exit Sub

        Dim positionList = ctpNet.GetPositions()

        Log("持仓信息数量=" & positionList.Count.ToString())

        For Each pi In positionList
            Log("持仓信息:instrumentID=" & pi.InstrumentID & ",direction=" & pi.Direction.ToString() & ",position=" & pi.Position.ToString())
        Next
    End Sub

    ''' <summary>
    ''' 获取指定合约持仓
    ''' </summary>
    Private Sub Button_GetPositionInfoById_Click(sender As Object, e As EventArgs) Handles Button_GetPosition.Click
        If CheckConnection() = False Then Exit Sub

        Dim instrumentID = GetInstrumentID()
        If String.IsNullOrWhiteSpace(instrumentID) Then
            Log("请输入合约代码")
            Exit Sub
        End If

        Dim direction = GetDirection()
        If direction = 0 Then
            Log("请选择买卖方向")
            Exit Sub
        End If

        '获取投机持仓
        Dim hedgeFlag = CtpNetHedgeFlagType.Speculation

        Dim pi = ctpNet.GetPosition(instrumentID, direction, hedgeFlag)
        If IsNothing(pi) Then
            Log(instrumentID & " 获取指定持仓失败")
        Else
            Log(instrumentID & " 持仓=" & pi.Position.ToString())
        End If
    End Sub

#End Region

#Region "接口事件"

    ''' <summary>
    ''' 行情服务器已连接
    ''' </summary>
    Private Sub ctpNet_OnMarketDataServerConnected() Handles CtpNet.OnMarketDataServerConnected
        LogEvent("行情服务器已连接")
    End Sub

    ''' <summary>
    ''' 行情服务器已断开
    ''' </summary>
    Private Sub ctpNet_OnMarketDataServerDisconnected(pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnMarketDataServerDisconnected
        LogEvent("行情服务器已断开")
    End Sub

    ''' <summary>
    ''' 交易服务器已连接
    ''' </summary>
    Private Sub ctpNet_OnTradeServerConnected() Handles CtpNet.OnTradeServerConnected
        LogEvent("交易服务器已连接")
    End Sub

    ''' <summary>
    ''' 交易服务器已断开
    ''' </summary>
    Private Sub ctpNet_OnTradeServerDisconnected(pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnTradeServerDisconnected
        LogEvent("交易服务器已断开")
    End Sub

    ''' <summary>
    ''' 初始化完成
    ''' </summary>
    Private Sub ctpNet_OnAccountInitializeResponse() Handles CtpNet.OnInitialized
        LogEvent("初始化完成")
    End Sub

    ''' <summary>
    ''' 组件清理响应
    ''' </summary>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnReleased(pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnReleased
        LogEvent("组件清理响应:" & pRspInfo.ResponseMsg())
    End Sub

    ''' <summary>
    ''' 交易通知
    ''' </summary>
    ''' <param name="pNoticeInfo"></param>
    Private Sub ctpNet_OnTradingNotice(pNoticeInfo As CtpNetTradingNoticeInfoField) Handles CtpNet.OnTradingNotice
        If IsNothing(pNoticeInfo) Then Exit Sub
        LogEvent("交易通知:" & pNoticeInfo.FieldContent)
    End Sub

    ''' <summary>
    ''' 程序日志
    ''' </summary>
    Private Sub ctpNet_OnLog(pField As CtpNetLogInfoFieldEx) Handles CtpNet.OnLog

        If CheckBox_ShowOnLog.Checked = False Then Exit Sub

        LogEvent("[日志]" & pField.LogContent)

    End Sub

    ''' <summary>
    ''' 合约状态通知
    ''' </summary>
    Private Sub ctpNet_OnInstrumentStatus(pField As CtpNetInstrumentStatusField) Handles CtpNet.OnInstrumentStatus

        If CheckBox_ShowOnInstrumentStatus.Checked = False Then Exit Sub

        LogEvent("合约状态通知,exchangeId=" & pField.ExchangeID & ",instrumentID=" & pField.InstrumentID & ",instrumentStatus=" & pField.InstrumentStatus.ToNameString())

    End Sub

    ''' <summary>
    ''' 资金变化事件
    ''' </summary>
    Private Sub ctpNet_OnAccountMoney(pField As CtpNetTradingAccountField) Handles CtpNet.OnTradingAccount

        If CheckBox_ShowOnAccountMoney.Checked = False Then Exit Sub

        LogEvent("资金变化,帐号=" & pField.AccountID & ",balance=" & pField.Balance.ToString() & ",available=" & pField.Available.ToString())
    End Sub

    ''' <summary>
    ''' 行情通知
    ''' </summary>
    Private Sub ctpNet_OnMarketData(pField As CtpNetDepthMarketDataField) Handles CtpNet.OnMarketData

        If CheckBox_ShowOnMarketData.Checked = False Then Exit Sub

        LogEvent("行情通知,instrumentID=" & pField.InstrumentID & ",lastPrice=" & pField.LastPrice.ToString() & ",updateTime=" & pField.UpdateTime)

    End Sub

    ''' <summary>
    ''' 报单通知
    ''' </summary>
    Private Sub ctpNet_OnOrder(pField As CtpNetOrderFieldEx) Handles CtpNet.OnOrder

        If CheckBox_ShowOnOrder.Checked = False Then Exit Sub

        '报单失败
        If pField.OrderSubmitStatus = CtpNetOrderSubmitStatusType.InsertRejected Then
            If CheckBox_ShowOnOrderFailed.Checked = False Then Exit Sub
            LogEvent("报单失败,帐号=" & pField.InvestorID & ",orderSn=" & pField.OrderSn.ToString() & " " & pField.StatusMsg)
            Exit Sub
        End If

        '撤单失败
        If pField.OrderSubmitStatus = CtpNetOrderSubmitStatusType.ModifyRejected Then
            If CheckBox_ShowOnOrderFailed.Checked = False Then Exit Sub
            LogEvent("撤单失败,帐号=" & pField.InvestorID & ",orderSn=" & pField.OrderSn.ToString() & " " & pField.StatusMsg)
            Exit Sub
        End If

        LogEvent("报单通知,帐号=" & pField.AccountID & ",orderSn=" & pField.OrderSn.ToString() & ",instrumentID=" & pField.InstrumentID & ",statusMsg=" & pField.StatusMsg)

    End Sub

    ''' <summary>
    ''' 成交通知
    ''' </summary>
    Private Sub ctpNet_OnTrade(pField As CtpNetTradeFieldEx) Handles CtpNet.OnTrade

        If CheckBox_ShowOnTrade.Checked = False Then Exit Sub

        LogEvent("成交通知,帐号=" & pField.InvestorID & ",orderSn=" & pField.OrderSn.ToString() & ",instrumentID=" & pField.InstrumentID)

    End Sub

    ''' <summary>
    ''' 持仓变化
    ''' </summary>
    Private Sub ctpNet_OnPosition(pField As CtpNetInvestorPositionFieldEx) Handles CtpNet.OnPosition

        If CheckBox_ShowOnPosition.Checked = False Then Exit Sub

        LogEvent("持仓变化,帐号=" & pField.InvestorID & ",instrumentID=" & pField.InstrumentID & ",position=" & pField.Position.ToString())

    End Sub

    ''' <summary>
    ''' 更新资金密码响应
    ''' </summary>
    ''' <param name="pPasswordInfo"></param>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnUpdateTradingAccountPassword(pPasswordInfo As CtpNetTradingAccountPasswordUpdateField, pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnUpdateTradingAccountPassword
        If pRspInfo.IsError() Then
            LogEvent("更新资金密码失败:" & pRspInfo.ResponseMsg())
            Return
        End If

        LogEvent("更新资金密码成功")
    End Sub

    ''' <summary>
    ''' 更新交易密码响应
    ''' </summary>
    ''' <param name="pPasswordInfo"></param>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnUpdateUserPassword(pPasswordInfo As CtpNetUserPasswordUpdateField, pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnUpdateUserPassword
        If pRspInfo.IsError() Then
            LogEvent("更新交易密码失败:" & pRspInfo.ResponseMsg())
            Return
        End If

        LogEvent("更新交易密码成功")
    End Sub
    
    ''' <summary>
    ''' 银行资金转期货响应(入金)
    ''' </summary>
    ''' <param name="pTransferInfo"></param>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnFromBankToFuture(pTransferInfo As CtpNetRspTransferField, pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnFromBankToFuture
        If pRspInfo.IsError() Then
            LogEvent("银行资金转期货失败:" & pRspInfo.ResponseMsg())
            Return
        End If

        LogEvent("银行资金转期货成功,转帐金额=" & pTransferInfo.TradeAmount)
    End Sub

    ''' <summary>
    ''' 期货资金转银行响应(出金)
    ''' </summary>
    ''' <param name="pTransferInfo"></param>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnFromFutureToBank(pTransferInfo As CtpNetRspTransferField, pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnFromFutureToBank
        If pRspInfo.IsError() Then
            LogEvent("期货资金转银行失败:" & pRspInfo.ResponseMsg())
            Return
        End If

        LogEvent("期货资金转银行成功,转帐金额=" & pTransferInfo.TradeAmount)
    End Sub

    ''' <summary>
    ''' 查询银行余额响应
    ''' </summary>
    ''' <param name="pBankAccountInfo"></param>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnQueryBankAccountMoney(pBankAccountInfo As CtpNetNotifyQueryAccountField, pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnQueryBankAccountMoney
        If pRspInfo.IsError() Then
            LogEvent("查询银行余额失败:" & pRspInfo.ResponseMsg())
            Return
        End If

        LogEvent("银行余额:银行可用金额=" & pBankAccountInfo.BankUseAmount & ",银行可取金额=" & pBankAccountInfo.BankFetchAmount)
    End Sub

    ''' <summary>
    ''' 查询手续费率响应
    ''' </summary>
    ''' <param name="pCommissionRate"></param>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnQueryCommissionRate(pCommissionRate As CtpNetInstrumentCommissionRateField, pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnQueryCommissionRate
        If pRspInfo.IsError() Then
            LogEvent("查询手续费率失败:" & pRspInfo.ResponseMsg())
            Return
        End If

        If IsNothing(pCommissionRate) Then Exit Sub

        LogEvent("查询手续费率成功," & "InstrumentID=" & pCommissionRate.InstrumentID & ",OpenRatioByMoney=" & pCommissionRate.OpenRatioByMoney & ",OpenRatioByVolume=" & pCommissionRate.OpenRatioByVolume)
    End Sub

    ''' <summary>
    ''' 查询保证金率响应
    ''' </summary>
    ''' <param name="pMarginRate"></param>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnQueryMarginRate(pMarginRate As CtpNetInstrumentMarginRateField, pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnQueryMarginRate
        If pRspInfo.IsError() Then
            LogEvent("查询保证金率失败:" & pRspInfo.ResponseMsg())
            Return
        End If

        If IsNothing(pMarginRate) Then Exit Sub

        LogEvent("查询保证金率成功," & "InstrumentID=" & pMarginRate.InstrumentID & ",LongMarginRatioByMoney=" & pMarginRate.LongMarginRatioByMoney & ",LongMarginRatioByVolume=" & pMarginRate.LongMarginRatioByVolume)
    End Sub

    ''' <summary>
    ''' 查询报单手续费率响应
    ''' </summary>
    ''' <param name="pOrderCommissionRate"></param>
    ''' <param name="pRspInfo"></param>
    Private Sub ctpNet_OnQueryOrderCommissionRate(pOrderCommissionRate As CtpNetInstrumentOrderCommRateField, pRspInfo As CtpNetRspInfoFieldEx) Handles CtpNet.OnQueryOrderCommissionRate
        If pRspInfo.IsError() Then
            LogEvent("查询报单手续费率失败:" & pRspInfo.ResponseMsg())
            Return
        End If

        If IsNothing(pOrderCommissionRate) Then Exit Sub

        LogEvent("查询报单手续费率成功," & "InstrumentID=" & pOrderCommissionRate.InstrumentID & ",OrderCommByVolume=" & pOrderCommissionRate.OrderCommByVolume & ",OrderActionCommByVolume=" & pOrderCommissionRate.OrderActionCommByVolume)
    End Sub

#End Region

End Class
